@@ -1,7 +1,7 @@
 /* Test file for mpfr_atanh.
 
-Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Free Software Foundation, Inc.
-Contributed by the AriC and Caramel projects, INRIA.
+Copyright 2001-2020 Free Software Foundation, Inc.
+Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -17,12 +17,9 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdio.h>
-#include <stdlib.h>
-
 #include "mpfr-test.h"
 
 #define TEST_FUNCTION mpfr_atanh
@@ -78,14 +75,14 @@ special (void)
   /* atanh(+0) = +0, atanh(-0) = -0 */
   mpfr_set_ui (x, 0, MPFR_RNDN);
   mpfr_atanh (y, x, MPFR_RNDN);
-  if (mpfr_cmp_ui (y, 0) || mpfr_sgn (y) < 0)
+  if (MPFR_NOTZERO (y) || MPFR_IS_NEG (y))
     {
       printf ("Error: mpfr_atanh(+0) <> +0\n");
       exit (1);
     }
   mpfr_neg (x, x, MPFR_RNDN);
   mpfr_atanh (y, x, MPFR_RNDN);
-  if (mpfr_cmp_ui (y, 0) || mpfr_sgn (y) > 0)
+  if (MPFR_NOTZERO (y) || MPFR_IS_POS (y))
     {
       printf ("Error: mpfr_atanh(-0) <> -0\n");
       exit (1);
@@ -104,7 +101,7 @@ special (void)
   if (!mpfr_nan_p (y))
     {
       printf ("Error: mpfr_atanh(+Inf) <> NaN\n");
-      mpfr_print_binary (y); printf ("\n");
+      mpfr_dump (y);
       exit (1);
     }
 
@@ -162,7 +159,7 @@ special (void)
   if (mpfr_cmp (y, z))
     {
       printf ("Error: mpfr_atanh (3)\n");
-      mpfr_print_binary (y); printf ("\n");
+      mpfr_dump (y);
       exit (1);
     }
 
@@ -178,10 +175,10 @@ main (int argc, char *argv[])
 
   special ();
 
-  test_generic (2, 100, 25);
+  test_generic (MPFR_PREC_MIN, 100, 25);
 
   data_check ("data/atanh", mpfr_atanh, "mpfr_atanh");
-  bad_cases (mpfr_atanh, mpfr_tanh, "mpfr_atanh", 256, -128, 9,
+  bad_cases (mpfr_atanh, mpfr_tanh, "mpfr_atanh", 256, -128, 8,
              4, 128, 800, 100);
 
   tests_end_mpfr ();
